@@ -2,6 +2,7 @@ package main
 
 import (
 	"crypto/tls"
+	"encoding/hex"
 	"encoding/json"
 	"flag"
 	"fmt"
@@ -37,6 +38,8 @@ var (
 	Serial     = flag.Int("serial.value", 0, "Serial number")
 	Session    = flag.Int("session.id", 0, "Session ID")
 
+	FlagVersion = flag.Int("rtr.version", 2, "What RTR version you want to use, Version 2 is draft-ietf-sidrops-8210bis-10")
+
 	ConnType     = flag.String("type", "plain", "Type of connection: plain, tls or ssh")
 	ValidateCert = flag.Bool("tls.validate", true, "Validate TLS")
 
@@ -96,6 +99,41 @@ func (c *Client) HandlePDU(cs *rtr.ClientSession, pdu rtr.PDU) {
 		c.Data.Data = append(c.Data.Data, rj)
 		c.Data.Metadata.Counts++
 
+		if *LogDataPDU {
+			log.Debugf("Received: %v", pdu)
+		}
+	case *rtr.PDURouterKey:
+		skiHex := hex.EncodeToString(pdu.SubjectKeyIdentifier)
+		rj := prefixfile.BgpSecKeyJson{
+			Asn:    uint32(pdu.ASN),
+			Pubkey: pdu.SubjectPublicKeyInfo,
+			Ski:    skiHex,
+		}
+		c.Data.BgpSecKeys = append(c.Data.BgpSecKeys, rj)
+
+		if *LogDataPDU {
+			log.Debugf("Received: %v", pdu)
+		}
+
+	case *rtr.PDUASPA:
+		if c.Data.ASPA == nil {
+			c.Data.ASPA = &prefixfile.ProviderAuthorizationsJson{
+				IPv4: make([]prefixfile.ASPAJson, 0),
+				IPv6: make([]prefixfile.ASPAJson, 0),
+			}
+		}
+		aj := prefixfile.ASPAJson{
+			CustomerAsid: pdu.CustomerASNumber,
+			Providers:    pdu.ProviderASNumbers,
+		}
+
+		switch pdu.AFIFlags {
+		case rtr.AFI_IPv4:
+			c.Data.ASPA.IPv4 = append(c.Data.ASPA.IPv4, aj)
+		case rtr.AFI_IPv6:
+			c.Data.ASPA.IPv6 = append(c.Data.ASPA.IPv6, aj)
+		}
+
 		if *LogDataPDU {
 			log.Debugf("Received: %v", pdu)
 		}
@@ -133,11 +171,21 @@ func main() {
 		os.Exit(0)
 	}
 
+	targetVersion := rtr.PROTOCOL_VERSION_0
+	if *FlagVersion > 2 {
+		log.Fatalf("Invalid RTR Version provided, the highest version this release supports is 2")
+	}
+	if *FlagVersion == 1 {
+		targetVersion = rtr.PROTOCOL_VERSION_1
+	} else if *FlagVersion == 2 {
+		targetVersion = rtr.PROTOCOL_VERSION_2
+	}
+
 	lvl, _ := log.ParseLevel(*LogLevel)
 	log.SetLevel(lvl)
 
 	cc := rtr.ClientConfiguration{
-		ProtocolVersion: rtr.PROTOCOL_VERSION_1,
+		ProtocolVersion: uint8(targetVersion),
 		Log:             log.StandardLogger(),
 	}
 
@@ -202,6 +250,9 @@ func main() {
 	log.Infof("Connecting with %v to %v", *ConnType, *Connect)
 	err := clientSession.Start(*Connect, typeToId[*ConnType], configTLS, configSSH)
 	if err != nil {
+		if err == io.EOF && targetVersion == rtr.PROTOCOL_VERSION_2 {
+			log.Warnf("EOF From remote side, This might be due to version 2 being requested, try using -rtr.version 1")
+		}
 		log.Fatal(err)
 	}
 
