@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"crypto/tls"
 	_ "embed"
+	"encoding/hex"
 	"encoding/json"
 	"errors"
 	"flag"
@@ -198,11 +199,11 @@ func (t *thresholds) Set(value string) error {
 	return nil
 }
 
-func decodeJSON(data []byte) (*prefixfile.VRPList, error) {
+func decodeJSON(data []byte) (*prefixfile.RPKIList, error) {
 	buf := bytes.NewBuffer(data)
 	dec := json.NewDecoder(buf)
 
-	var vrplistjson prefixfile.VRPList
+	var vrplistjson prefixfile.RPKIList
 	err := dec.Decode(&vrplistjson)
 	return &vrplistjson, err
 }
@@ -345,7 +346,7 @@ func (c *Client) Start(id int, ch chan int) {
 					continue
 				}
 
-				updatedVrpMap, inGracePeriod = BuildNewVrpMap(log.WithField("client", c.id), c.vrps, decoded.Data, tCurrentUpdate)
+				updatedVrpMap, inGracePeriod = BuildNewVrpMap(log.WithField("client", c.id), c.vrps, decoded, tCurrentUpdate)
 			}
 
 			VRPInGracePeriod.With(prometheus.Labels{"url": c.Path}).Set(float64(inGracePeriod))
@@ -368,7 +369,8 @@ func (c *Client) Start(id int, ch chan int) {
 //   * contains all the VRPs in newVRPs
 //   * keeps the firstSeen value for VRPs already in the old map
 //   * keeps elements around for GracePeriod after they are not in the input.
-func BuildNewVrpMap(log *log.Entry, currentVrps VRPMap, newVrps []prefixfile.VRPJson, now time.Time) (VRPMap, int) {
+func BuildNewVrpMap(log *log.Entry, currentVrps VRPMap, pfxFile *prefixfile.RPKIList, now time.Time) (VRPMap, int) {
+	var newVrps = pfxFile.ROA
 	tCurrentUpdate := now.Unix()
 	res := make(VRPMap)
 
@@ -404,6 +406,27 @@ func BuildNewVrpMap(log *log.Entry, currentVrps VRPMap, newVrps []prefixfile.VRP
 		}
 	}
 
+	for _, bgpsecKey := range pfxFile.BgpSecKeys {
+		key := fmt.Sprintf("%s-%d-%s", bgpsecKey.Ski, bgpsecKey.Asn, bgpsecKey.Pubkey)
+
+		firstSeen := tCurrentUpdate
+		currentEntry, ok := currentVrps[key]
+		if ok && currentEntry.Visible {
+			// VRP is still visible, so keep the existing `FirstSeen`.
+			firstSeen = currentEntry.FirstSeen
+		}
+
+		copyOf := bgpsecKey
+
+		res[key] = &VRPJsonSimple{
+			ASN:        bgpsecKey.Asn,
+			FirstSeen:  firstSeen,
+			LastSeen:   tCurrentUpdate,
+			Visible:    true,
+			BGPSecData: &copyOf,
+		}
+	}
+
 	// Copy objects that are within the grace period to the new map
 	gracePeriodEnds := tCurrentUpdate - int64(GracePeriod.Seconds())
 	inGracePeriod := 0
@@ -482,6 +505,28 @@ func (c *Client) HandlePDU(cs *rtr.ClientSession, pdu rtr.PDU) {
 			delete(c.vrpsRtr, key)
 		}
 
+		c.compRtrLock.Unlock()
+	case *rtr.PDURouterKey:
+		vrp := VRPJsonSimple{
+			ASN:       pdu.ASN,
+			FirstSeen: time.Now().Unix(),
+			Visible:   true,
+			BGPSecData: &prefixfile.BgpSecKeyJson{
+				Asn:    pdu.ASN,
+				Pubkey: pdu.SubjectPublicKeyInfo,
+				Ski:    hex.EncodeToString(pdu.SubjectKeyIdentifier),
+			},
+		}
+
+		key := fmt.Sprintf("%s-%d-%s", vrp.BGPSecData.Ski, pdu.ASN, pdu.SubjectPublicKeyInfo)
+		c.compRtrLock.Lock()
+
+		if pdu.Flags == rtr.FLAG_ADDED {
+			c.vrpsRtr[key] = &vrp
+		} else {
+			delete(c.vrpsRtr, key)
+		}
+
 		c.compRtrLock.Unlock()
 	case *rtr.PDUEndOfData:
 		log.Infof("%d: Received: %v", c.id, pdu)
@@ -678,12 +723,13 @@ type diffMetadata struct {
 }
 
 type VRPJsonSimple struct {
-	ASN       uint32 `json:"asn"`
-	Length    uint8  `json:"max-length"`
-	Prefix    string `json:"prefix"`
-	FirstSeen int64  `json:"first-seen"`
-	LastSeen  int64  `json:"last-seen"`
-	Visible   bool   `json:"visible"`
+	ASN        uint32                    `json:"asn"`
+	Length     uint8                     `json:"max-length"`
+	Prefix     string                    `json:"prefix"`
+	FirstSeen  int64                     `json:"first-seen"`
+	LastSeen   int64                     `json:"last-seen"`
+	Visible    bool                      `json:"visible"`
+	BGPSecData *prefixfile.BgpSecKeyJson `json:"bgpsec,omitempty"`
 }
 type VRPMap map[string]*VRPJsonSimple
 
