@@ -10,8 +10,8 @@ import (
 	"errors"
 	"flag"
 	"fmt"
-	"net"
 	"net/http"
+	"net/netip"
 	"os"
 	"os/signal"
 	"sort"
@@ -77,7 +77,7 @@ var (
 
 	TimeCheck = flag.Bool("checktime", true, "Check if JSON file isn't stale (disable by passing -checktime=false)")
 
-	CacheBin = flag.String("cache", "https://console.rpki-client.org/vrps.json", "URL of the cached JSON data")
+	CacheBin = flag.String("cache", "https://console.rpki-client.org/rpki.json", "URL of the Validated RPKI data in JSON format")
 
 	Etag            = flag.Bool("etag", true, "Control usage of Etag header (disable with -etag=false)")
 	LastModified    = flag.Bool("last.modified", true, "Control usage of Last-Modified header (disable with -last.modified=false)")
@@ -97,16 +97,16 @@ var (
 	NumberOfVRPs = prometheus.NewGaugeVec(
 		prometheus.GaugeOpts{
 			Name: "rpki_vrps",
-			Help: "Number of VRPs.",
+			Help: "Number of VRPs by source and status.",
 		},
 		[]string{"ip_version", "filtered", "path"},
 	)
-	NumberOfBRKs = prometheus.NewGaugeVec(
+	NumberOfObjects = prometheus.NewGaugeVec(
 		prometheus.GaugeOpts{
-			Name: "rpki_bgpsec",
-			Help: "Number of BGPsec keys.",
+			Name: "rpki_objects",
+			Help: "Number of RPKI objects (in cache) by type",
 		},
-		[]string{},
+		[]string{"type"},
 	)
 	LastRefresh = prometheus.NewGaugeVec(
 		prometheus.GaugeOpts{
@@ -143,6 +143,12 @@ var (
 		},
 		[]string{"type"},
 	)
+	CurrentSerial = prometheus.NewGauge(
+		prometheus.GaugeOpts{
+			Name: "rtr_serial",
+			Help: "Current serial.",
+		},
+	)
 
 	protoverToLib = map[int]uint8{
 		0: rtr.PROTOCOL_VERSION_0,
@@ -152,13 +158,14 @@ var (
 )
 
 func initMetrics() {
+	prometheus.MustRegister(NumberOfObjects)
 	prometheus.MustRegister(NumberOfVRPs)
-	prometheus.MustRegister(NumberOfBRKs)
 	prometheus.MustRegister(LastChange)
 	prometheus.MustRegister(LastRefresh)
 	prometheus.MustRegister(RefreshStatusCode)
 	prometheus.MustRegister(ClientsMetric)
 	prometheus.MustRegister(PDUsRecv)
+	prometheus.MustRegister(CurrentSerial)
 }
 
 func metricHTTP() {
@@ -173,18 +180,18 @@ func newSHA256(data []byte) []byte {
 	return hash[:]
 }
 
-func decodeJSON(data []byte) (*prefixfile.VRPList, error) {
+func decodeJSON(data []byte) (*prefixfile.RPKIList, error) {
 	buf := bytes.NewBuffer(data)
 	dec := json.NewDecoder(buf)
 
-	var vrplistjson prefixfile.VRPList
-	err := dec.Decode(&vrplistjson)
-	return &vrplistjson, err
+	var rpkilistjson prefixfile.RPKIList
+	err := dec.Decode(&rpkilistjson)
+	return &rpkilistjson, err
 }
 
-func isValidPrefixLength(prefix *net.IPNet, maxLength uint8) bool {
-	plen, max := net.IPMask.Size(prefix.Mask)
-
+func isValidPrefixLength(prefix netip.Prefix, maxLength uint8) bool {
+	plen := prefix.Bits()
+	max := prefix.Addr().BitLen()
 	if plen == 0 || uint8(plen) > maxLength || maxLength > uint8(max) {
 		log.Errorf("%s Maxlength wrong: %d - %d", prefix, plen, maxLength)
 		return false
@@ -200,15 +207,14 @@ func isValidPrefixLength(prefix *net.IPNet, maxLength uint8) bool {
 // Will return a deduped slice, as well as total VRPs, IPv4 VRPs, IPv6 VRPs, BGPsec Keys and ASPA records
 func processData(vrplistjson []prefixfile.VRPJson,
 	brklistjson []prefixfile.BgpSecKeyJson,
-	aspajson *prefixfile.ProviderAuthorizationsJson) /*Export*/ ([]rtr.VRP, []rtr.BgpsecKey, []rtr.VAP, int, int, int) {
-	//
-	filterDuplicates := make(map[string]bool)
+	aspajson []prefixfile.VAPJson) /*Export*/ ([]rtr.VRP, []rtr.BgpsecKey, []rtr.VAP, int, int) {
+	filterDuplicates := make(map[string]struct{})
 
 	// It may be tempting to change this to a simple time.Since() but that will
 	// grab the current time every time it's invoked, time calls can be slow on
 	// some platforms, so lets just get the unix time when we start and use that
 	// to compare it all
-	NowUnix := time.Now().UTC().Unix()
+	NowUnix := time.Now().Unix()
 
 	var vrplist []rtr.VRP
 	var brklist = make([]rtr.BgpsecKey, 0)
@@ -235,12 +241,12 @@ func processData(vrplistjson []prefixfile.VRPJson,
 		if v.Expires != nil {
 			// Prevent stale VRPs from being considered
 			// https://github.com/bgp/stayrtr/issues/15
-			if int(NowUnix) > int(*v.Expires) {
+			if NowUnix > *v.Expires {
 				continue
 			}
 		}
 
-		if prefix.IP.To4() != nil {
+		if prefix.Addr().Is4() {
 			countv4++
 		} else {
 			countv6++
@@ -251,21 +257,52 @@ func processData(vrplistjson []prefixfile.VRPJson,
 		if exists {
 			continue
 		}
-		filterDuplicates[key] = true
+		filterDuplicates[key] = struct{}{}
 
 		vrp := rtr.VRP{
-			Prefix: *prefix,
+			Prefix: prefix,
 			ASN:    asn,
 			MaxLen: v.Length,
 		}
 		vrplist = append(vrplist, vrp)
 	}
 
+	sort.Slice(vrplist, func(i, j int) bool {
+		// Sort VRPs as per draft-ietf-sidrops-8210bis-10
+		/*
+			11. ROA PDU Race Minimization
+				When a cache is sending ROA (IPv4 or IPv6) PDUs to a router, especially an initial
+				full load in response to a Reset Query PDU, two undesirable race conditions are possible:
+
+			Break Before Make:
+				For some prefix P, an AS may announce two (or more) ROAs because they are in the
+				process of changing what provider AS is announcing P. This is a case of "make before break."
+				If a cache is feeding a router and sends the one not yet in service a significant time
+				before sending the one currently in service, then BGP data could be marked invalid during
+				the interval. To minimize that interval, the cache SHOULD announce all ROAs for the same
+				prefix as close to sequentially as possible.
+			Shorter Prefix First:
+				If an AS has issued a ROA for P0, and another AS (likely their customer) has issued a ROA
+				for P1 which is a sub-prefix of P0, a router which receives the ROA for P0 before that for
+				P1 is likely to mark a BGP prefix P1 invalid. Therefore, the cache SHOULD announce the
+				sub-prefix P1 before the covering prefix P0.
+		*/
+
+		if vrplist[i].Prefix.Bits() == vrplist[j].Prefix.Bits() {
+			if vrplist[i].MaxLen != vrplist[j].MaxLen {
+				return vrplist[i].MaxLen > vrplist[j].MaxLen
+			}
+			return vrplist[i].Prefix.Addr().Compare(vrplist[j].Prefix.Addr()) < 1
+		} else {
+			return vrplist[i].Prefix.Bits() > vrplist[j].Prefix.Bits()
+		}
+	})
+
 	for _, v := range brklistjson {
 		if v.Expires != nil {
 			// Prevent stale VRPs from being considered
 			// https://github.com/bgp/stayrtr/issues/15
-			if int(NowUnix) > int(*v.Expires) {
+			if NowUnix > *v.Expires {
 				continue
 			}
 		}
@@ -282,35 +319,26 @@ func processData(vrplistjson []prefixfile.VRPJson,
 		})
 	}
 
-	if aspajson != nil {
-		aspalist = handleASPAList(aspajson.IPv4, NowUnix, aspalist, rtr.AFI_IPv4)
-		aspalist = handleASPAList(aspajson.IPv6, NowUnix, aspalist, rtr.AFI_IPv6)
-	}
-
-	return vrplist, brklist, aspalist, countv4 + countv6, countv4, countv6
-}
-
-func handleASPAList(list []prefixfile.ASPAJson, NowUnix int64, aspalist []rtr.VAP, AFI uint8) []rtr.VAP {
-	for _, v := range list {
+	for _, v := range aspajson {
 		if v.Expires != nil {
-			if int(NowUnix) > int(*v.Expires) {
+			if NowUnix > *v.Expires {
 				continue
 			}
 		}
 
 		// Ensure that these are sorted, otherwise they
-		// don't has right.
+		// don't hash right.
 		sort.Slice(v.Providers, func(i, j int) bool {
 			return v.Providers[i] < v.Providers[j]
 		})
 
 		aspalist = append(aspalist, rtr.VAP{
-			AFI:         AFI,
 			CustomerASN: v.CustomerAsid,
 			Providers:   v.Providers,
 		})
 	}
-	return aspalist
+
+	return vrplist, brklist, aspalist, countv4, countv6
 }
 
 type IdenticalFile struct {
@@ -321,13 +349,13 @@ func (e IdenticalFile) Error() string {
 	return fmt.Sprintf("File %s is identical to the previous version", e.File)
 }
 
-var errVRPJsonFileTooOld = errors.New("VRP JSON file is older than 24 hours")
+var errRPKIJsonFileTooOld = errors.New("RPKI JSON file is older than 24 hours")
 
 // Update the state based on the current slurm file and data.
 func (s *state) updateFromNewState() error {
 	sessid := s.server.GetSessionId()
 
-	vrpsjson := s.lastdata.Data
+	vrpsjson := s.lastdata.ROA
 	if vrpsjson == nil {
 		return nil
 	}
@@ -337,15 +365,12 @@ func (s *state) updateFromNewState() error {
 	}
 	aspajson := s.lastdata.ASPA
 	if aspajson == nil {
-		aspajson = &prefixfile.ProviderAuthorizationsJson{
-			IPv4: make([]prefixfile.ASPAJson, 0),
-			IPv6: make([]prefixfile.ASPAJson, 0),
-		}
+		aspajson = make([]prefixfile.VAPJson, 0)
 	}
 
 	buildtime, err := time.Parse(time.RFC3339, s.lastdata.Metadata.Buildtime)
 	if s.lastdata.Metadata.GeneratedUnix != nil {
-		buildtime, err = time.Unix(int64(*s.lastdata.Metadata.GeneratedUnix), 0), nil
+		buildtime, err = time.Unix(*s.lastdata.Metadata.GeneratedUnix, 0), nil
 	}
 	if s.checktime {
 		if err != nil {
@@ -353,18 +378,19 @@ func (s *state) updateFromNewState() error {
 		}
 		notafter := buildtime.Add(time.Hour * 24)
 		if time.Now().UTC().After(notafter) {
-			log.Warnf("VRP JSON file is older than 24 hours: %v", buildtime)
-			return errVRPJsonFileTooOld
+			log.Warnf("RPKI JSON file is older than 24 hours: %v", buildtime)
+			return errRPKIJsonFileTooOld
 		}
 	}
 
 	if s.slurm != nil {
-		vrpsjson, aspajson.IPv4, aspajson.IPv6, bgpsecjson = s.slurm.FilterAssert(vrpsjson, aspajson.IPv4, aspajson.IPv6, bgpsecjson, log.StandardLogger())
+		vrpsjson, aspajson, bgpsecjson = s.slurm.FilterAssert(vrpsjson, aspajson, bgpsecjson, log.StandardLogger())
 	}
 
-	vrps, brks, vaps, count, countv4, countv6 := processData(vrpsjson, bgpsecjson, aspajson)
+	vrps, brks, vaps, countv4, countv6 := processData(vrpsjson, bgpsecjson, aspajson)
+	count := len(vrps) + len(brks) + len(vaps)
 
-	log.Infof("New update (%v uniques, %v total prefixes).", len(vrps), count)
+	log.Infof("New update (%v uniques, %v total prefixes, %v vaps, %v router keys).", len(vrps), count, len(vaps), len(brks))
 	return s.applyUpdateFromNewState(vrps, brks, vaps, sessid, vrpsjson, bgpsecjson, aspajson, countv4, countv6)
 }
 
@@ -372,7 +398,7 @@ func (s *state) updateFromNewState() error {
 func (s *state) reloadFromCurrentState() error {
 	sessid := s.server.GetSessionId()
 
-	vrpsjson := s.lastdata.Data
+	vrpsjson := s.lastdata.ROA
 	if vrpsjson == nil {
 		return nil
 	}
@@ -382,15 +408,12 @@ func (s *state) reloadFromCurrentState() error {
 	}
 	aspajson := s.lastdata.ASPA
 	if aspajson == nil {
-		aspajson = &prefixfile.ProviderAuthorizationsJson{
-			IPv4: make([]prefixfile.ASPAJson, 0),
-			IPv6: make([]prefixfile.ASPAJson, 0),
-		}
+		aspajson = make([]prefixfile.VAPJson, 0)
 	}
 
 	buildtime, err := time.Parse(time.RFC3339, s.lastdata.Metadata.Buildtime)
 	if s.lastdata.Metadata.GeneratedUnix != nil {
-		buildtime, err = time.Unix(int64(*s.lastdata.Metadata.GeneratedUnix), 0), nil
+		buildtime, err = time.Unix(*s.lastdata.Metadata.GeneratedUnix, 0), nil
 	}
 	if s.checktime {
 		if err != nil {
@@ -398,18 +421,19 @@ func (s *state) reloadFromCurrentState() error {
 		}
 		notafter := buildtime.Add(time.Hour * 24)
 		if time.Now().UTC().After(notafter) {
-			log.Warnf("VRP JSON file is older than 24 hours: %v", buildtime)
-			return errVRPJsonFileTooOld
+			log.Warnf("RPKI JSON file is older than 24 hours: %v", buildtime)
+			return errRPKIJsonFileTooOld
 		}
 	}
 
 	if s.slurm != nil {
-		vrpsjson, aspajson.IPv4, aspajson.IPv6, bgpsecjson = s.slurm.FilterAssert(vrpsjson, aspajson.IPv4, aspajson.IPv6, bgpsecjson, log.StandardLogger())
+		vrpsjson, aspajson, bgpsecjson = s.slurm.FilterAssert(vrpsjson, aspajson, bgpsecjson, log.StandardLogger())
 	}
 
-	vrps, brks, vaps, count, countv4, countv6 := processData(vrpsjson, bgpsecjson, aspajson)
-	if s.server.CountVRPs() != count {
-		log.Infof("New update to old state (%v uniques, %v total prefixes). (old %v - new %v)", len(vrps), count, s.server.CountVRPs(), count)
+	vrps, brks, vaps, countv4, countv6 := processData(vrpsjson, bgpsecjson, aspajson)
+	count := len(vrps) + len(brks) + len(vaps)
+	if s.server.CountSDs() != count {
+		log.Infof("New update to old state (%v uniques, %v total prefixes). (old %v - new %v)", len(vrps), count, s.server.CountSDs(), count)
 		return s.applyUpdateFromNewState(vrps, brks, vaps, sessid, vrpsjson, bgpsecjson, aspajson, countv4, countv6)
 	}
 	return nil
@@ -417,10 +441,10 @@ func (s *state) reloadFromCurrentState() error {
 
 func (s *state) applyUpdateFromNewState(vrps []rtr.VRP, brks []rtr.BgpsecKey, vaps []rtr.VAP,
 	sessid uint16,
-	vrpsjson []prefixfile.VRPJson, brksjson []prefixfile.BgpSecKeyJson, aspajson *prefixfile.ProviderAuthorizationsJson,
+	vrpsjson []prefixfile.VRPJson, brksjson []prefixfile.BgpSecKeyJson, aspajson []prefixfile.VAPJson,
 	countv4 int, countv6 int) error {
 
-	SDs := make([]rtr.SendableData, 0)
+	SDs := make([]rtr.SendableData, 0, len(vrps)+len(brks)+len(vaps))
 	for _, v := range vrps {
 		SDs = append(SDs, v.Copy())
 	}
@@ -430,22 +454,26 @@ func (s *state) applyUpdateFromNewState(vrps []rtr.VRP, brks []rtr.BgpsecKey, va
 	for _, v := range vaps {
 		SDs = append(SDs, v.Copy())
 	}
-	s.server.AddData(SDs)
+	if !s.server.AddData(SDs) {
+		log.Info("No difference to current cache")
+		return nil
+	}
 
 	serial, _ := s.server.GetCurrentSerial(sessid)
-	log.Infof("Updated added, new serial %v", serial)
+	log.Infof("Update added, new serial %v", serial)
 	if s.sendNotifs {
 		log.Debugf("Sending notifications to clients")
 		s.server.NotifyClientsLatest()
 	}
+	CurrentSerial.Set(float64(serial))
 
 	s.lockJson.Lock()
-	s.exported = prefixfile.VRPList{
+	s.exported = prefixfile.RPKIList{
 		Metadata: prefixfile.MetaData{
 			Counts:    len(vrpsjson),
 			Buildtime: s.lastdata.Metadata.Buildtime,
 		},
-		Data:       vrpsjson,
+		ROA:        vrpsjson,
 		BgpSecKeys: brksjson,
 		ASPA:       aspajson,
 	}
@@ -456,13 +484,13 @@ func (s *state) applyUpdateFromNewState(vrps []rtr.VRP, brks []rtr.BgpsecKey, va
 		var countv4_dup int
 		var countv6_dup int
 		for _, vrp := range vrps {
-			if vrp.Prefix.IP.To4() != nil {
+			if vrp.Prefix.Addr().Is4() {
 				countv4_dup++
-			} else if vrp.Prefix.IP.To16() != nil {
+			} else if vrp.Prefix.Addr().Is6() {
 				countv6_dup++
 			}
 		}
-		s.metricsEvent.UpdateMetrics(countv4, countv6, countv4_dup, countv6_dup, s.lastchange, s.lastts, *CacheBin, len(brks))
+		s.metricsEvent.UpdateMetrics(countv4, countv6, countv4_dup, countv6_dup, s.lastchange, s.lastts, *CacheBin, len(brks), len(vaps))
 	}
 
 	return nil
@@ -484,23 +512,23 @@ func (s *state) updateFile(file string) (bool, error) {
 	}
 
 	hsum := newSHA256(data)
-	if s.lasthash != nil {
-		cres := bytes.Compare(s.lasthash, hsum)
+	if s.lasthashCache != nil {
+		cres := bytes.Compare(s.lasthashCache, hsum)
 		if cres == 0 {
 			return false, IdenticalFile{File: file}
 		}
 	}
 
-	log.Infof("new cache file: Updating sha256 hash %x -> %x", s.lasthash, hsum)
+	log.Debugf("new cache file: Updating sha256 hash %x -> %x", s.lasthashCache, hsum)
+	s.lasthashCache = hsum
 
-	vrplistjson, err := decodeJSON(data)
+	rpkilistjson, err := decodeJSON(data)
 	if err != nil {
 		return false, err
 	}
 
-	s.lasthash = hsum
 	s.lastchange = time.Now().UTC()
-	s.lastdata = vrplistjson
+	s.lastdata = rpkilistjson
 
 	return true, nil
 }
@@ -518,6 +546,16 @@ func (s *state) updateSlurm(file string) (bool, error) {
 		RefreshStatusCode.WithLabelValues(file, fmt.Sprintf("%d", code)).Inc()
 	}
 
+	hsum := newSHA256(data)
+	if s.lasthashSlurm != nil {
+		cres := bytes.Compare(s.lasthashSlurm, hsum)
+		if cres == 0 {
+			return false, IdenticalFile{File: file}
+		}
+	}
+	log.Debugf("new slurm file: Updating sha256 hash %x -> %x", s.lasthashCache, hsum)
+	s.lasthashSlurm = hsum
+
 	buf := bytes.NewBuffer(data)
 
 	slurm, err := prefixfile.DecodeJSONSlurm(buf)
@@ -602,7 +640,7 @@ func (s *state) routineUpdate(file string, interval int, slurmFile string) {
 		if cacheUpdated || slurmNotPresentOrUpdated {
 			err := s.updateFromNewState()
 			if err != nil {
-				if err == errVRPJsonFileTooOld {
+				if err == errRPKIJsonFileTooOld {
 					// If the exiting build time is over 24 hours, It's time to drop everything out.
 					// to avoid routing on stale data
 					buildTime := s.exported.Metadata.GetBuildTime()
@@ -630,11 +668,12 @@ func (s *state) exporter(wr http.ResponseWriter, r *http.Request) {
 }
 
 type state struct {
-	lastdata   *prefixfile.VRPList
-	lasthash   []byte
-	lastchange time.Time
-	lastts     time.Time
-	sendNotifs bool
+	lastdata      *prefixfile.RPKIList
+	lasthashCache []byte
+	lasthashSlurm []byte
+	lastchange    time.Time
+	lastts        time.Time
+	sendNotifs    bool
 
 	fetchConfig *utils.FetchConfig
 
@@ -642,7 +681,7 @@ type state struct {
 
 	metricsEvent *metricsEvent
 
-	exported prefixfile.VRPList
+	exported prefixfile.RPKIList
 	lockJson *sync.RWMutex
 
 	slurm *prefixfile.SlurmConfig
@@ -671,12 +710,16 @@ func (m *metricsEvent) HandlePDU(c *rtr.Client, pdu rtr.PDU) {
 				"_", -1))).Inc()
 }
 
-func (m *metricsEvent) UpdateMetrics(numIPv4 int, numIPv6 int, numIPv4filtered int, numIPv6filtered int, changed time.Time, refreshed time.Time, file string, brkCount int) {
+func (m *metricsEvent) UpdateMetrics(numIPv4 int, numIPv6 int, numIPv4filtered int, numIPv6filtered int, changed time.Time, refreshed time.Time, file string, brkCount int, aspaCount int) {
+	NumberOfObjects.WithLabelValues("vaps").Set(float64(aspaCount))
+	NumberOfObjects.WithLabelValues("bgpsec_pubkeys").Set(float64(brkCount))
+	NumberOfObjects.WithLabelValues("vrps").Set(float64(numIPv4 + numIPv6))
+	NumberOfObjects.WithLabelValues("effective_vrps").Set(float64(numIPv4filtered + numIPv6filtered))
+
 	NumberOfVRPs.WithLabelValues("ipv4", "filtered", file).Set(float64(numIPv4filtered))
 	NumberOfVRPs.WithLabelValues("ipv4", "unfiltered", file).Set(float64(numIPv4))
 	NumberOfVRPs.WithLabelValues("ipv6", "filtered", file).Set(float64(numIPv6filtered))
 	NumberOfVRPs.WithLabelValues("ipv6", "unfiltered", file).Set(float64(numIPv6))
-	NumberOfBRKs.WithLabelValues().Set(float64(brkCount))
 	LastChange.WithLabelValues(file).Set(float64(changed.UnixNano() / 1e9))
 }
 
@@ -735,7 +778,7 @@ func run() error {
 
 	s := state{
 		server:       server,
-		lastdata:     &prefixfile.VRPList{},
+		lastdata:     &prefixfile.RPKIList{},
 		metricsEvent: me,
 		sendNotifs:   *SendNotifs,
 		checktime:    *TimeCheck,
