@@ -8,12 +8,12 @@ import (
 	"github.com/stretchr/testify/assert"
 )
 
-func GenerateVrps(size uint32, offset uint32) []VRP {
-	vrps := make([]VRP, size)
+func GenerateVrps(size uint32, offset uint32) []SendableData {
+	vrps := make([]SendableData, size)
 	for i := uint32(0); i < size; i++ {
 		ipFinal := make([]byte, 4)
 		binary.BigEndian.PutUint32(ipFinal, i+offset)
-		vrps[i] = VRP{
+		vrps[i] = &VRP{
 			Prefix: net.IPNet{
 				IP:   net.IP(append([]byte{0xfd, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, ipFinal...)),
 				Mask: net.IPMask([]byte{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}),
@@ -82,13 +82,22 @@ func TestComputeDiff(t *testing.T) {
 			ASN:    65002,
 		},
 	}
-	added, removed, unchanged := ComputeDiff(newVrps, prevVrps)
+
+	newVrpsSD, prevVrpsAsSD := make([]SendableData, 0), make([]SendableData, 0)
+	for _, v := range newVrps {
+		newVrpsSD = append(newVrpsSD, v.Copy())
+	}
+	for _, v := range prevVrps {
+		prevVrpsAsSD = append(prevVrpsAsSD, v.Copy())
+	}
+
+	added, removed, unchanged := ComputeDiff(newVrpsSD, prevVrpsAsSD)
 	assert.Len(t, added, 1)
 	assert.Len(t, removed, 1)
 	assert.Len(t, unchanged, 1)
-	assert.Equal(t, added[0].ASN, uint32(65003))
-	assert.Equal(t, removed[0].ASN, uint32(65001))
-	assert.Equal(t, unchanged[0].ASN, uint32(65002))
+	assert.Equal(t, added[0].(*VRP).ASN, uint32(65003))
+	assert.Equal(t, removed[0].(*VRP).ASN, uint32(65001))
+	assert.Equal(t, unchanged[0].(*VRP).ASN, uint32(65002))
 }
 
 func TestApplyDiff(t *testing.T) {
@@ -186,19 +195,70 @@ func TestApplyDiff(t *testing.T) {
 			Flags:  FLAG_REMOVED,
 		},
 	}
-	vrps := ApplyDiff(diff, prevVrps)
+	diffSD, prevVrpsAsSD := make([]SendableData, 0), make([]SendableData, 0)
+	for _, v := range diff {
+		diffSD = append(diffSD, v.Copy())
+	}
+	for _, v := range prevVrps {
+		prevVrpsAsSD = append(prevVrpsAsSD, v.Copy())
+	}
+
+	vrps := ApplyDiff(diffSD, prevVrpsAsSD)
 
 	assert.Len(t, vrps, 6)
-	assert.Equal(t, vrps[0].ASN, uint32(65001))
-	assert.Equal(t, vrps[0].Flags, uint8(FLAG_ADDED))
-	assert.Equal(t, vrps[1].ASN, uint32(65005))
-	assert.Equal(t, vrps[1].Flags, uint8(FLAG_REMOVED))
-	assert.Equal(t, vrps[2].ASN, uint32(65003))
-	assert.Equal(t, vrps[2].Flags, uint8(FLAG_ADDED))
-	assert.Equal(t, vrps[3].ASN, uint32(65004))
-	assert.Equal(t, vrps[3].Flags, uint8(FLAG_REMOVED))
-	assert.Equal(t, vrps[4].ASN, uint32(65006))
-	assert.Equal(t, vrps[4].Flags, uint8(FLAG_REMOVED))
-	assert.Equal(t, vrps[5].ASN, uint32(65007))
-	assert.Equal(t, vrps[5].Flags, uint8(FLAG_ADDED))
+	assert.Equal(t, vrps[0].(*VRP).ASN, uint32(65001))
+	assert.Equal(t, vrps[0].(*VRP).GetFlag(), uint8(FLAG_ADDED))
+	assert.Equal(t, vrps[1].(*VRP).ASN, uint32(65005))
+	assert.Equal(t, vrps[1].(*VRP).GetFlag(), uint8(FLAG_REMOVED))
+	assert.Equal(t, vrps[2].(*VRP).ASN, uint32(65003))
+	assert.Equal(t, vrps[2].(*VRP).GetFlag(), uint8(FLAG_ADDED))
+	assert.Equal(t, vrps[3].(*VRP).ASN, uint32(65004))
+	assert.Equal(t, vrps[3].(*VRP).GetFlag(), uint8(FLAG_REMOVED))
+	assert.Equal(t, vrps[4].(*VRP).ASN, uint32(65006))
+	assert.Equal(t, vrps[4].(*VRP).GetFlag(), uint8(FLAG_REMOVED))
+	assert.Equal(t, vrps[5].(*VRP).ASN, uint32(65007))
+	assert.Equal(t, vrps[5].(*VRP).GetFlag(), uint8(FLAG_ADDED))
+}
+
+func TestComputeDiffBGPSEC(t *testing.T) {
+	newVrps := []BgpsecKey{
+		{
+			ASN:    65003,
+			Pubkey: []byte("hurr"),
+			Ski:    []byte("durr"),
+		},
+		{
+			Pubkey: []byte("abc"),
+			Ski:    []byte("dce"),
+			ASN:    65002,
+		},
+	}
+	prevVrps := []BgpsecKey{
+		{
+			Pubkey: []byte("murr"),
+			Ski:    []byte("durr"),
+			ASN:    65001,
+		},
+		{
+			Pubkey: []byte("abc"),
+			Ski:    []byte("dce"),
+			ASN:    65002,
+		},
+	}
+
+	newVrpsSD, prevVrpsAsSD := make([]SendableData, 0), make([]SendableData, 0)
+	for _, v := range newVrps {
+		newVrpsSD = append(newVrpsSD, v.Copy())
+	}
+	for _, v := range prevVrps {
+		prevVrpsAsSD = append(prevVrpsAsSD, v.Copy())
+	}
+
+	added, removed, unchanged := ComputeDiff(newVrpsSD, prevVrpsAsSD)
+	assert.Len(t, added, 1)
+	assert.Len(t, removed, 1)
+	assert.Len(t, unchanged, 1)
+	assert.Equal(t, added[0].(*BgpsecKey).ASN, uint32(65003))
+	assert.Equal(t, removed[0].(*BgpsecKey).ASN, uint32(65001))
+	assert.Equal(t, unchanged[0].(*BgpsecKey).ASN, uint32(65002))
 }
