@@ -7,7 +7,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"net"
+	"net/netip"
 )
 
 type Logger interface {
@@ -23,6 +23,7 @@ const (
 
 	PROTOCOL_VERSION_0 = 0
 	PROTOCOL_VERSION_1 = 1
+	PROTOCOL_VERSION_2 = 2
 
 	PDU_ID_SERIAL_NOTIFY  = 0
 	PDU_ID_SERIAL_QUERY   = 1
@@ -34,6 +35,7 @@ const (
 	PDU_ID_CACHE_RESET    = 8
 	PDU_ID_ROUTER_KEY     = 9
 	PDU_ID_ERROR_REPORT   = 10
+	PDU_ID_ASPA           = 11
 
 	FLAG_ADDED   = 1
 	FLAG_REMOVED = 0
@@ -47,6 +49,9 @@ const (
 	PDU_ERROR_WITHDRAWUNKNOWN = 6
 	PDU_ERROR_DUPANNOUNCE     = 7
 
+	AFI_IPv4 = uint8(0)
+	AFI_IPv6 = uint8(1)
+
 	TYPE_UNKNOWN = iota
 	TYPE_PLAIN
 	TYPE_TLS
@@ -84,6 +89,8 @@ func TypeToString(t uint8) string {
 		return "Router Key"
 	case PDU_ID_ERROR_REPORT:
 		return "Error Report"
+	case PDU_ID_ASPA:
+		return "ASPA PDU"
 	default:
 		return fmt.Sprintf("Unknown type %d", t)
 	}
@@ -242,16 +249,15 @@ func (pdu *PDUCacheResponse) Write(wr io.Writer) {
 }
 
 type PDUIPv4Prefix struct {
-	Version uint8
-	Prefix  net.IPNet
-	MaxLen  uint8
+	Prefix  netip.Prefix
 	ASN     uint32
+	Version uint8
+	MaxLen  uint8
 	Flags   uint8
 }
 
 func (pdu *PDUIPv4Prefix) String() string {
-	mask, _ := pdu.Prefix.Mask.Size()
-	return fmt.Sprintf("PDU IPv4 Prefix v%d %s/%d(->/%d), origin: AS%d, flags: %d", pdu.Version, pdu.Prefix.IP.String(), mask, pdu.MaxLen, pdu.ASN, pdu.Flags)
+	return fmt.Sprintf("PDU IPv4 Prefix v%d %s(->/%d), origin: AS%d, flags: %d", pdu.Version, pdu.Prefix.String(), pdu.MaxLen, pdu.ASN, pdu.Flags)
 }
 
 func (pdu *PDUIPv4Prefix) Bytes() []byte {
@@ -273,30 +279,28 @@ func (pdu *PDUIPv4Prefix) GetType() uint8 {
 }
 
 func (pdu *PDUIPv4Prefix) Write(wr io.Writer) {
-	mask, _ := pdu.Prefix.Mask.Size()
 	binary.Write(wr, binary.BigEndian, uint8(pdu.Version))
 	binary.Write(wr, binary.BigEndian, uint8(PDU_ID_IPV4_PREFIX))
 	binary.Write(wr, binary.BigEndian, uint16(0))
 	binary.Write(wr, binary.BigEndian, uint32(20))
 	binary.Write(wr, binary.BigEndian, pdu.Flags)
-	binary.Write(wr, binary.BigEndian, uint8(mask))
+	binary.Write(wr, binary.BigEndian, uint8(pdu.Prefix.Bits()))
 	binary.Write(wr, binary.BigEndian, pdu.MaxLen)
 	binary.Write(wr, binary.BigEndian, uint8(0))
-	binary.Write(wr, binary.BigEndian, pdu.Prefix.IP.To4())
+	binary.Write(wr, binary.BigEndian, pdu.Prefix.Addr().As4())
 	binary.Write(wr, binary.BigEndian, pdu.ASN)
 }
 
 type PDUIPv6Prefix struct {
-	Version uint8
-	Prefix  net.IPNet
-	MaxLen  uint8
+	Prefix  netip.Prefix
 	ASN     uint32
+	Version uint8
+	MaxLen  uint8
 	Flags   uint8
 }
 
 func (pdu *PDUIPv6Prefix) String() string {
-	mask, _ := pdu.Prefix.Mask.Size()
-	return fmt.Sprintf("PDU IPv6 Prefix v%d %s/%d(->/%d), origin: AS%d, flags: %d", pdu.Version, pdu.Prefix.IP.String(), mask, pdu.MaxLen, pdu.ASN, pdu.Flags)
+	return fmt.Sprintf("PDU IPv6 Prefix v%d %s(->/%d), origin: AS%d, flags: %d", pdu.Version, pdu.Prefix.String(), pdu.MaxLen, pdu.ASN, pdu.Flags)
 }
 
 func (pdu *PDUIPv6Prefix) Bytes() []byte {
@@ -318,16 +322,15 @@ func (pdu *PDUIPv6Prefix) GetType() uint8 {
 }
 
 func (pdu *PDUIPv6Prefix) Write(wr io.Writer) {
-	mask, _ := pdu.Prefix.Mask.Size()
 	binary.Write(wr, binary.BigEndian, uint8(pdu.Version))
 	binary.Write(wr, binary.BigEndian, uint8(PDU_ID_IPV6_PREFIX))
 	binary.Write(wr, binary.BigEndian, uint16(0))
 	binary.Write(wr, binary.BigEndian, uint32(32))
 	binary.Write(wr, binary.BigEndian, pdu.Flags)
-	binary.Write(wr, binary.BigEndian, uint8(mask))
+	binary.Write(wr, binary.BigEndian, uint8(pdu.Prefix.Bits()))
 	binary.Write(wr, binary.BigEndian, pdu.MaxLen)
 	binary.Write(wr, binary.BigEndian, uint8(0))
-	binary.Write(wr, binary.BigEndian, pdu.Prefix.IP.To16())
+	binary.Write(wr, binary.BigEndian, pdu.Prefix.Addr().As16())
 	binary.Write(wr, binary.BigEndian, pdu.ASN)
 }
 
@@ -372,7 +375,7 @@ func (pdu *PDUEndOfData) Write(wr io.Writer) {
 	if pdu.Version == PROTOCOL_VERSION_0 {
 		binary.Write(wr, binary.BigEndian, uint32(12))
 		binary.Write(wr, binary.BigEndian, pdu.SerialNumber)
-	} else if pdu.Version == PROTOCOL_VERSION_1 {
+	} else if pdu.Version == PROTOCOL_VERSION_1 || pdu.Version == PROTOCOL_VERSION_2 {
 		binary.Write(wr, binary.BigEndian, uint32(24))
 		binary.Write(wr, binary.BigEndian, pdu.SerialNumber)
 		binary.Write(wr, binary.BigEndian, pdu.RefreshInterval)
@@ -417,9 +420,9 @@ func (pdu *PDUCacheReset) Write(wr io.Writer) {
 type PDURouterKey struct {
 	Version              uint8
 	Flags                uint8
-	SubjectKeyIdentifier [20]byte
+	SubjectKeyIdentifier []byte
 	ASN                  uint32
-	SubjectPublicKeyInfo uint32
+	SubjectPublicKeyInfo []byte
 }
 
 func (pdu *PDURouterKey) String() string {
@@ -445,14 +448,18 @@ func (pdu *PDURouterKey) GetType() uint8 {
 }
 
 func (pdu *PDURouterKey) Write(wr io.Writer) {
+	if len(pdu.SubjectKeyIdentifier) != 20 {
+		return
+	}
+
 	binary.Write(wr, binary.BigEndian, uint8(pdu.Version))
 	binary.Write(wr, binary.BigEndian, uint8(PDU_ID_ROUTER_KEY))
 	binary.Write(wr, binary.BigEndian, uint8(pdu.Flags))
 	binary.Write(wr, binary.BigEndian, uint8(0))
-	binary.Write(wr, binary.BigEndian, uint32(36))
-	binary.Write(wr, binary.BigEndian, pdu.SubjectKeyIdentifier)
+	binary.Write(wr, binary.BigEndian, uint32(32+len(pdu.SubjectPublicKeyInfo)))
+	wr.Write(pdu.SubjectKeyIdentifier)
 	binary.Write(wr, binary.BigEndian, pdu.ASN)
-	binary.Write(wr, binary.BigEndian, pdu.SubjectPublicKeyInfo)
+	wr.Write(pdu.SubjectPublicKeyInfo)
 }
 
 type PDUErrorReport struct {
@@ -505,6 +512,51 @@ func (pdu *PDUErrorReport) Write(wr io.Writer) {
 	}
 }
 
+type PDUASPA struct {
+	Version           uint8
+	Flags             uint8
+	AFIFlags          uint8
+	ProviderASCount   uint16
+	CustomerASNumber  uint32
+	ProviderASNumbers []uint32
+}
+
+func (pdu *PDUASPA) String() string {
+	return fmt.Sprintf("PDU ASPA v%d TODO", pdu.Version) // TODO
+}
+
+func (pdu *PDUASPA) Bytes() []byte {
+	b := bytes.NewBuffer([]byte{})
+	pdu.Write(b)
+	return b.Bytes()
+}
+
+func (pdu *PDUASPA) SetVersion(version uint8) {
+	pdu.Version = version
+}
+
+func (pdu *PDUASPA) GetVersion() uint8 {
+	return pdu.Version
+}
+
+func (pdu *PDUASPA) GetType() uint8 {
+	return PDU_ID_ASPA
+}
+
+func (pdu *PDUASPA) Write(wr io.Writer) {
+	binary.Write(wr, binary.BigEndian, uint8(pdu.Version))
+	binary.Write(wr, binary.BigEndian, uint8(PDU_ID_ASPA))
+	binary.Write(wr, binary.BigEndian, uint16(0))
+	binary.Write(wr, binary.BigEndian, uint32(16+(len(pdu.ProviderASNumbers)*4)))
+	binary.Write(wr, binary.BigEndian, uint8(pdu.Flags))
+	binary.Write(wr, binary.BigEndian, uint8(pdu.AFIFlags))
+	binary.Write(wr, binary.BigEndian, uint16(pdu.ProviderASCount))
+	binary.Write(wr, binary.BigEndian, uint32(pdu.CustomerASNumber))
+	for _, pasn := range pdu.ProviderASNumbers {
+		binary.Write(wr, binary.BigEndian, uint32(pasn))
+	}
+}
+
 func DecodeBytes(b []byte) (PDU, error) {
 	buf := bytes.NewBuffer(b)
 	return Decode(buf)
@@ -589,9 +641,9 @@ func Decode(rdr io.Reader) (PDU, error) {
 		}
 		prefixLen := int(toread[1])
 		ip := toread[4:8]
-		ipnet := net.IPNet{
-			IP:   ip,
-			Mask: net.CIDRMask(prefixLen, 32),
+		addr, ok := netip.AddrFromSlice(ip)
+		if !ok {
+			return nil, fmt.Errorf("ip slice length is not 4 or 16: %+v", addr)
 		}
 		asn := binary.BigEndian.Uint32(toread[8:])
 		return &PDUIPv4Prefix{
@@ -599,7 +651,7 @@ func Decode(rdr io.Reader) (PDU, error) {
 			Flags:   uint8(toread[0]),
 			MaxLen:  uint8(toread[2]),
 			ASN:     asn,
-			Prefix:  ipnet,
+			Prefix:  netip.PrefixFrom(addr, prefixLen),
 		}, nil
 	case PDU_ID_IPV6_PREFIX:
 		if len(toread) != 24 {
@@ -607,9 +659,9 @@ func Decode(rdr io.Reader) (PDU, error) {
 		}
 		prefixLen := int(toread[1])
 		ip := toread[4:20]
-		ipnet := net.IPNet{
-			IP:   ip,
-			Mask: net.CIDRMask(prefixLen, 128),
+		addr, ok := netip.AddrFromSlice(ip)
+		if !ok {
+			return nil, fmt.Errorf("ip slice length is not 4 or 16: %+v", addr)
 		}
 		asn := binary.BigEndian.Uint32(toread[20:])
 		return &PDUIPv6Prefix{
@@ -617,7 +669,7 @@ func Decode(rdr io.Reader) (PDU, error) {
 			Flags:   uint8(toread[0]),
 			MaxLen:  uint8(toread[2]),
 			ASN:     asn,
-			Prefix:  ipnet,
+			Prefix:  netip.PrefixFrom(addr, prefixLen),
 		}, nil
 	case PDU_ID_END_OF_DATA:
 		if len(toread) != 4 && len(toread) != 16 {
@@ -653,16 +705,18 @@ func Decode(rdr io.Reader) (PDU, error) {
 			Version: pver,
 		}, nil
 	case PDU_ID_ROUTER_KEY:
-		if len(toread) != 28 {
-			return nil, fmt.Errorf("wrong length for Router Key PDU: %d < 8", len(toread))
+		if len(toread) < 28 {
+			return nil, fmt.Errorf("wrong length for Router Key PDU: %d < 28", len(toread))
 		}
 		asn := binary.BigEndian.Uint32(toread[20:24])
-		spki := binary.BigEndian.Uint32(toread[24:28])
-		ski := [20]byte{}
+		spki := toread[24:]
+		ski := make([]byte, 20)
 		copy(ski[:], toread[0:20])
 		return &PDURouterKey{
 			Version:              pver,
 			SubjectKeyIdentifier: ski,
+			// Router Key uses a rarely used spot that is also used by the SessionID, So we we will just bitshift
+			Flags:                uint8(sessionId >> 8),
 			ASN:                  asn,
 			SubjectPublicKeyInfo: spki,
 		}, nil
@@ -688,6 +742,35 @@ func Decode(rdr io.Reader) (PDU, error) {
 			PDUCopy:   errPdu,
 			ErrorMsg:  errMsg,
 		}, nil
+	case PDU_ID_ASPA:
+		if len(toread) < 8 {
+			return nil, fmt.Errorf("wrong length for ASPA PDU: %d < 16", len(toread))
+		}
+
+		aspaFlag := uint8(toread[0])
+		aspaAFIFlag := uint8(toread[1])
+		PASCount := binary.BigEndian.Uint16(toread[2:4])
+		CASN := binary.BigEndian.Uint32(toread[4:8])
+
+		PASNs := make([]uint32, 0)
+		rbuf := bytes.NewReader(toread[8:])
+		for i := 0; i < int(PASCount); i++ {
+			var asn uint32
+			err := binary.Read(rbuf, binary.BigEndian, &asn)
+			if err != nil {
+				return nil, err
+			}
+			PASNs = append(PASNs, asn)
+		}
+
+		return &PDUASPA{
+			Version:           pver,
+			Flags:             aspaFlag,
+			AFIFlags:          aspaAFIFlag,
+			ProviderASCount:   PASCount,
+			CustomerASNumber:  CASN,
+			ProviderASNumbers: PASNs,
+		}, nil
 	default:
 		return nil, errors.New("could not decode packet")
 	}
